#!/usr/bin/python
# -*- coding: utf-8 -*-
#
"""
    Generic utilities classes.
    
    Name: main.py
    Author: Alessandro dos Santos Ferreira ( santosferreira.alessandro@gmail.com )
"""
import cv2
import random
import time

class ColorUtils(object):
    """Set of utilities to manipulate colors."""

    @staticmethod
    def rand_color():
        """Return a random color.
        
        Returns
        -------
        color : string
            Color in format #FFFFFF.
        """
        r = random.randint
        
        return '#%02X%02X%02X' % (r(0, 255), r(0, 255), r(0, 255))
        

class ImageUtils(object):
    """Set of utilities to manipulate images."""
    
    @staticmethod
    def image_to_bgr(image):
        """Convert a image to BGR.
        
        Parameters
        ----------
        image : opencv RGB color image
            Image to be converted.
        
        Returns
        -------
        image : opencv BGR color image
            Image converted to BGR.
        """
        return cv2.cvtColor(image, cv2.COLOR_RGB2BGR)
        
    @staticmethod
    def image_to_hsv(image, bgr = False):
        """Convert a image to HSV.
        
        Parameters
        ----------
        image : opencv 3-channel color image
            Image to be converted.
        bgr : boolean, optional, default = False
            If true image is BGR, else image is RGB.
        
        Returns
        -------
        image : opencv HSV color image
            Image converted to HSV.
        """
        if bgr == False:
            image = ImageUtils.image_to_bgr(image)
        return cv2.cvtColor(image, cv2.COLOR_BGR2HSV)
        
    @staticmethod
    def image_to_cielab(image, bgr = False):
        """Convert a image to CIELab.
        
        Parameters
        ----------
        image : opencv 3-channel color image
            Image to be converted.
        bgr : boolean, optional, default = False
            If true image is BGR, else image is RGB.
        
        Returns
        -------
        image : opencv CIELab color image
            Image converted to CIELab.
        """
        if bgr == False:
            image = ImageUtils.image_to_bgr(image)
        return cv2.cvtColor(image, cv2.COLOR_BGR2LAB)
        
    @staticmethod
    def image_grayscale(image, bgr = False):
        """Convert a image to grayscale.
        
        Parameters
        ----------
        image : opencv 3-channel color image
            Image to be converted.
        bgr : boolean, optional, default = False
            If true image is BGR, else image is RGB.
        
        Returns
        -------
        image : opencv grayscale image
            Image converted to grayscale.
        """
        if bgr == False:
            image = ImageUtils.image_to_bgr(image)
        return cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
        
    @staticmethod
    def image_binary(image, bgr = False, grayscale = False):
        """Convert a image to binary.
        
        Parameters
        ----------
        image : opencv 3-channel color image
            Image to be converted.
        bgr : boolean, optional, default = False
            If true image is BGR, else image is RGB.
        grayscale : boolean, optional, default = False
            If true image is grayscale, else image is 3-channel color.
        
        Returns
        -------
        image : opencv binary image
            Image converted to binary.
        """
        if grayscale == False:
            image = ImageUtils.image_grayscale(image, bgr)
        
        image = cv2.GaussianBlur(image,(5,5),0)
        _, image = cv2.threshold(image, 0, 255, cv2.THRESH_BINARY+cv2.THRESH_OTSU)
        return image
        
    @staticmethod
    def image_edge_detector(image, canny_min = 100, canny_max = 200, bgr = False, grayscale = False):
        """Detect the edges of image.
        
        Parameters
        ----------
        image : opencv 3-channel color image
            Source image to be analyzed.
        canny_min : integer, optional, default = 100
            First threshold for the hysteresis procedure.
        canny_max : integer, optional, default = 200
            Second threshold for the hysteresis procedure.
        bgr : boolean, optional, default = False
            If true image is BGR, else image is RGB.
        grayscale : boolean, optional, default = False
            If true image is grayscale, else image is 3-channel color.
        
        Returns
        -------
        image : opencv grayscale image
            Image with edges detected.
        """
        if grayscale == False:
            image = ImageUtils.image_grayscale(image, bgr)
        return cv2.Canny(image, canny_min, canny_max)
        
    @staticmethod
    def image_resize(image, width, height):
        """Resize the image.
        
        Parameters
        ----------
        image : opencv image
            Image to be resized.
        width : integer
            Width of image.
        height : integer
            Height of image.
        
        Returns
        -------
        image : opencv image
            Image resized.
        """
        return cv2.resize(image, (width, height))

        
class TimeUtils(object):
    """Set of utilities to manipulate time."""

    @staticmethod
    def get_time():
        """Return the current time.
        
        Returns
        -------
        time : float
            Returns the time as a floating point number expressed in seconds since the epoch, in UTC.
        """
        return time.time()
